#ifndef _3DENGINE_H_
#define _3DENGINE_H_

#include <stdlib.h>
#include <vector>
#include <iostream.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "matrix.h"
#include "vector.h"
#include "import.h"
#include "system.h"

class color
{
public:
	unsigned char       r, g, b;

	inline void set()
	{
		glColor3b(r, g, b);
	}
};

class image
{
private:
	static GLuint       CurrentTexture;


public:
	GLuint              TextureIndex;      // Texture index common to OpenGL

	color*              Data;
	size_t              Width;
	size_t              Height;

	void                load(const char* FileName);
	void                load(i_image& ImportImage);

	static inline void set(image* Image)
	{
		if(!Image)
			glBindTexture(GL_TEXTURE_2D, CurrentTexture = 0);
		else if(CurrentTexture != Image->TextureIndex)
			glBindTexture(GL_TEXTURE_2D, CurrentTexture = Image->TextureIndex);
	}

	inline void draw(float x1, float y1, float x2, float y2)
	{
		glBegin(GL_POLYGON);
		glTexCoord2f(0, 0);
		glVertex3f(x1, y1, 0.5f);
		glTexCoord2f(0.99f, 0);
		glVertex3f(x2, y1, 0.5f);
		glTexCoord2f(0.99f, 0.99f);
		glVertex3f(x2, y2, 0.5f);
		glTexCoord2f(0, 0.99f);
		glVertex3f(x1, y2, 0.5f);
		glEnd();
	}
};

class vertex
{
public:
	bil::vector         Normal;
	float               x, y, z;           // 3D coordinate
	float               u, v;              // Colormap coordinate

	inline void set()
	{
		glTexCoord2f(u, v);
		glVertex3f(x, y, z);
	}
};

class polygon
{
public:
	vertex**            Vertices;
	size_t              VertexCount;

	image*              Colormap;

	bil::vector         Normal;
	float               Distance;

	inline void set()
	{
		if(Colormap)
			image::set(Colormap);

		glBegin(GL_POLYGON);
		for(size_t i = 0; i < VertexCount; i++)
			Vertices[i]->set();
		glEnd();
	}
};

class map_coord
{
public:
	float               u, v;
};

class map
{
private:
	size_t*             SortOrder;
public:
	vertex*             Vertices;
	size_t              VertexCount;

	polygon*            Faces;
	size_t              FaceCount;

	void                optimize();
	void                render();
};

class manager
{
public:
	std::vector<image*>  Images;
	std::vector<map*>    Maps;

	manager();
	~manager();

	void                 load_image(const char* FileName);
	void                 load_map(const char* FileName);
};

extern manager Manager;

#endif

